# Log4jScan.ps1 - run the Qualys Log4jScanner utility
#
# This script runs the Log4j vulnerablity scanner from Qualys and processes the results.
# If vulnerabilities are found, the scanner output file is renamed to Log4jScan_FAILED.txt.
# If an error occurs, it is trapped and written to Log4jScan_ERROR.txt.
#
# This script is run by the Kaseya procedure "Log4j Scan" which then checks for either of 
# the two files above and generates an alert through Kaseya that results in a CW ticket.
#
# Tom Strickland, Sequoya Technologies Group, 12-20-21
#
try
{
	#
	# check the PowerShell version first
	#
	if ($PSVersionTable.PSVersion.Major -lt 3) {throw "Script requires PowerShell v3 or later."}
	
	#
	# set to our working directory and clean up any previous output files
	#
	cd C:\Sequoya
	if (Test-Path Temp\log4jscan.txt) {Remove-Item -path Temp\log4jscan.txt}
	if (Test-Path Temp\log4jscan_FAILED.txt) {Remove-Item -path Temp\log4jscan_FAILED.txt}
	if (Test-Path Temp\log4jscan_ERROR.txt) {Remove-Item -path Temp\log4jscan_ERROR.txt}
	
	#
	# run the scan and output to a json file
	#
	Tools\Log4jScanner.exe /scan /report_pretty >Temp\log4jscan.txt
	
	#
	# the output file coding is wonky so we have to clean that up to remove the BOM
	# at the front and all of the extra NUL characters.  Put the result in the string $outs
	#
	$file = get-content .\Temp\log4jscan.txt -encoding byte -ErrorAction Stop
	$out = @()
	for (($i = 2) ; $i -lt $file.Count ; ++$i) { if ($file[$i] -ne 0) {$out += $file[$i]}}
	$outs = [system.text.encoding]::ASCII.GetString($out)
	
	#
	# parse the json and get the vulnerablity count. Write the result to Log4jScan_FAILED
	# if the count is >0
	#
	$jscan = ConvertFrom-Json -InputObject $outs -ErrorAction Stop
	if ($jscan.scanSummary.vulnerabilitiesFound -gt 0) 
	{
		Set-Content temp\Log4jScan_FAILED.txt -value $outs
	}
}
catch
{
	#
	# something bad happened. Write the error to Log4jScan_ERROR
	#
	write-host 'Exception caught'
	Set-Content temp\Log4jScan_ERROR.txt -value $_
}
